package np.santaclaus.warehouse;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;

public class HsqlDbSklad implements Sklad {
	private JdbcTemplate jdbcTemplate = JdbcTemplateFactory.getJdbcTemplate();

	private DarcekDao darcekDao = Registry.getDarcekDao();
	
	private VypocetDobrotyDietataStrategy vypocetDobrotyDietataStrategy = new VsetkyDetiSuDobreStrategy();
	
	public void prijmi(ListJeziskovi list) {
		/*
		 * Vlozit (INSERTnut) dieta a
		 * 		 overit, ci nahodou uz tam nie je
		 * Vlozit kazdy darcek
		 * Vlozit samotny list
		 */
		Dieta dieta = list.getOdosielatel();
		
		//FIXME duplicitny kod s HsqlDbDietaDao.uloz()
		SimpleJdbcInsert dietaInsert = new SimpleJdbcInsert(jdbcTemplate);
		dietaInsert.setTableName("DIETA");
		Map<String, Object> mapa = new HashMap<String, Object>();
		mapa.put("MENO", dieta.getMeno());
		dietaInsert.setGeneratedKeyName("ID");
		Number dietaKey = dietaInsert.executeAndReturnKey(mapa);
		dieta.setId(dietaKey.longValue());
		
		
		SimpleJdbcInsert listInsert = new SimpleJdbcInsert(jdbcTemplate);
		listInsert.setTableName("LIST");
		listInsert.setGeneratedKeyName("ID");
		Map<String, Object> mapaList = new HashMap<String, Object>();
		mapaList.put("ID_DIETA", dietaKey.longValue());
		
		Number listKey = listInsert.executeAndReturnKey(mapaList);
		list.setId(listKey.longValue());
		
		
		for (Darcek darcek : list.getDarceky()) {
			vloz(darcek, list);
		}
	}
	
	private void vloz(Darcek darcek, ListJeziskovi list) {
		SimpleJdbcInsert darcekInsert = new SimpleJdbcInsert(jdbcTemplate);
		darcekInsert.setTableName("DARCEK");
		darcekInsert.setGeneratedKeyName("ID");
		Map<String, Object> mapa = new HashMap<String, Object>();
		mapa.put("NAZOV", darcek.getNazov());
		mapa.put("ID_LIST", list.getId());
		
		Number key = darcekInsert.executeAndReturnKey(mapa);
		darcek.setId(key.longValue());
	}

	public boolean obsahujeList(Dieta dieta) {
		String sql = "select count(*) from list where id_dieta = ?";
		return jdbcTemplate.queryForInt(sql, dieta.getId()) == 1;
	}

	public void zmaz(ListJeziskovi list) {
		for (Darcek darcek : list.getDarceky()) {
			darcekDao.zmaz(darcek);
		}
		
		jdbcTemplate.update("DELETE FROM list WHERE ID = ?", list.getId());
		
	}
	
	public List<Darcek> vyber(Dieta dieta) {
		String sql = "SELECT darcek.id, darcek.nazov"
					+ " FROM list"
					+ " JOIN darcek on darcek.id_list = list.id"
					+ " WHERE id_dieta = ?";	
		//BeanPropertyRowMapper<Darcek> rowMapper = new BeanPropertyRowMapper<Darcek>(Darcek.class);
		DarcekRowMapper rowMapper = new DarcekRowMapper();
		List<Darcek> darceky = jdbcTemplate.query(sql, rowMapper, dieta.getId());
	
		return filtruj(darceky, vypocetDobrotyDietataStrategy, dieta);
	}
	
	private List<Darcek> filtruj(List<Darcek> darceky, VypocetDobrotyDietataStrategy strategia, Dieta dieta) {
		double celkovaDlzka = darceky.size();
		int ziadanaDlzka = (int) (celkovaDlzka * (strategia.vypocitajDobrotu(dieta) / 100.0));
		return darceky.subList(0, ziadanaDlzka);
	}
	
	public void setVypocetDobrotyDietataStrategy(
			VypocetDobrotyDietataStrategy vypocetDobrotyDietataStrategy) {
		this.vypocetDobrotyDietataStrategy = vypocetDobrotyDietataStrategy;
	}
	
	
	
	
	
	
	
	
	
}
